from tuprolog import logger

# noinspection PyUnresolvedReferences
import jpype
# noinspection PyUnresolvedReferences
import jpype.imports

# noinspection PyUnresolvedReferences
from it.unibo.tuprolog.solve import Solver

logger.debug("Loaded JVM classes from it.unibo.tuprolog.solve.*")
