from tuprolog import logger

# noinspection PyUnresolvedReferences
import jpype.imports

# noinspection PyUnresolvedReferences
from it.unibo.tuprolog.core.exception import TuPrologException
# noinspection PyUnresolvedReferences
from it.unibo.tuprolog.core.exception import SubstitutionException
# noinspection PyUnresolvedReferences
from it.unibo.tuprolog.core.exception import SubstitutionApplicationException

logger.debug("Loaded JVM classes from it.unibo.tuprolog.core.exception.*")
